@@ -1,24 +1,18 @@
 package WebService::Google::Language;
 
-use 5.006;
-
 use strict;
 use warnings;
 
-our $VERSION = '0.12_01';
+our $VERSION = '0.01';
 
-$VERSION = eval $VERSION;
-
-use Carp ();
+use Carp;
 use JSON 2.0 ();
 use LWP::UserAgent;
-use URI;
+use URI::Escape;
 
-use constant GOOGLE_DETECT_URL    => 'http://ajax.googleapis.com/ajax/services/language/detect';
-use constant GOOGLE_TRANSLATE_URL => 'http://ajax.googleapis.com/ajax/services/language/translate';
-use constant API_VERSION          => '1.0';
-use constant MAX_LENGTH           => 5000;
-use constant URL_MAX_LENGTH       => 2073;
+use constant GOOGLE_DETECT_URL    => 'http://ajax.googleapis.com/ajax/services/language/detect?v=1.0';
+use constant GOOGLE_TRANSLATE_URL => 'http://ajax.googleapis.com/ajax/services/language/translate?v=1.0';
+use constant MAX_LENGTH => 500;
 
 
 
@@ -28,37 +22,27 @@ use constant URL_MAX_LENGTH       => 2073;
 
 sub new {
   my $class = shift;
-  my $self = bless {}, $class;
-
   unshift @_, 'referer' if @_ % 2;
   my %conf = @_;
 
-  $self->referer(delete $conf{referer});
+  my $referer = delete $conf{'referer'};
+  croak "Constructor requires a non-empty parameter 'referer'"
+    unless defined $referer and $referer =~ /\S/;
 
-  for (qw'src dest key') {
+  my $self = { 'json' => JSON->new };
+
+  for (qw(src dest key)) {
     if (defined(my $value = delete $conf{$_})) {
       $self->{$_} = $value;
     }
   }
 
-  for (qw'json ua') {
-    if (defined(my $value = delete $conf{$_})) {
-      $self->$_($value);
-    }
-  }
-  unless ($self->json) {
-    $self->json(JSON->new);
-  }
-  unless ($self->ua) {
-    $conf{agent} = $class . ' ' . $class->VERSION
-      unless defined $conf{agent};
-    # respect proxy environment variables (reported by IZUT)
-    $conf{env_proxy} = 1
-      unless exists $conf{env_proxy};
-    $self->ua(LWP::UserAgent->new(%conf));
-  }
+  $conf{'agent'} = __PACKAGE__ . ' ' . $VERSION unless defined $conf{'agent'};
+  my $ua = LWP::UserAgent->new(%conf);
+  $ua->default_header('referer' => $referer);
+  $self->{'ua'} = $ua;
 
-  return $self;
+  bless $self, $class;
 }
 
 
@@ -71,151 +55,63 @@ sub translate {
   my $self = shift;
   unshift @_, 'text' if @_ % 2;
   my %args = @_;
-  my $src  = $args{src}  || $self->{src}  || '';
-  my $dest = $args{dest} || $self->{dest} || 'en';
-  return $self->_request($args{text}, $src . '|' . $dest);
+  my $src  = $args{'src'}  || $self->{'src'}  || '';
+  my $dest = $args{'dest'} || $self->{'dest'} || 'en';
+  return $self->_request($args{'text'}, $src . '%7C' . $dest);
 }
 
+
+
 sub detect {
   my $self = shift;
   unshift @_, 'text' if @_ % 2;
   my %args = @_;
-  return $self->_request($args{text});
+  return $self->_request($args{'text'});
 }
 
 *detect_language = \&detect;
 
+
+
 sub ping {
-  my $self = shift;
-  return $self->ua
-    ->get( GOOGLE_TRANSLATE_URL, referer => $self->referer )
-    ->is_success;
+  return $_[0]->{'ua'}->get(GOOGLE_TRANSLATE_URL)->is_success;
 }
 
 
 
 #
-#  accessors
-#
-
-sub json {
-  my $self = shift;
-  if (@_) {
-    my $json = shift;
-    Carp::croak q{'json' requires an object based on 'JSON'}
-      unless $json && $json->isa('JSON');
-    $self->{json} = $json;
-    return $self;
-  }
-  $self->{json};
-}
-
-sub referer {
-  my $self = shift;
-  if (@_) {
-    my $referer = shift;
-    unless (defined $referer && $referer =~ /\S/) {
-      my $name    = q{'referer'};
-      my $error   = 'requires a non-empty parameter';
-      my $caller  = (caller(1))[3];
-      Carp::croak $caller && $caller eq ref($self) . '::new'
-        ? "Constructor $error $name"
-        : "$name $error";
-    }
-    $self->{referer} = $referer;
-    return $self;
-  }
-  $self->{referer};
-}
-
-sub ua {
-  my $self = shift;
-  if (@_) {
-    my $ua = shift;
-    Carp::croak q{'ua' requires an object based on 'LWP::UserAgent'}
-      unless $ua && $ua->isa('LWP::UserAgent');
-    $self->{ua} = $ua;
-    return $self;
-  }
-  $self->{ua};
-}
-
-
-
-#
-#  private methods and functions
+#  private methods
 #
 
 sub _request {
   my ($self, $text, $langpair) = @_;
-  if (defined $text && $text =~ /\S/) {
-    _utf8_encode($text);
+  if (defined $text and $text =~ /\S/) {
+    utf8::encode($text);
     if (length $text > MAX_LENGTH) {
-      Carp::croak 'Google does not allow submission of text exceeding '
-        . MAX_LENGTH . ' characters in length';
+      croak "Google does not allow submission of text exceeding " . MAX_LENGTH . ' characters in length';
     }
   }
   else {
     return;
   }
 
-  my ($uri, $response);
-  my @param = ( v => API_VERSION );
-  push @param, key => $self->{key} if defined $self->{key};
-  if (defined $langpair) {
-    $uri = URI->new(GOOGLE_TRANSLATE_URL);
-    push @param, langpair => $langpair;
-  }
-  else {
-    $uri = URI->new(GOOGLE_DETECT_URL);
-  }
-  push @param, q => $text;
-  $uri->query_form(\@param);
-
-  my $length = length $uri;
-  if ($length > URL_MAX_LENGTH) {
-    if (defined $langpair) {
-      # POST only for translate
-      $uri->query_form( [] );
-      $response = $self->ua->post( $uri, \@param, referer => $self->referer );
-    }
-    else {
-      # detect can't be POSTed
-      Carp::croak "The length of the generated URL for this request is $length bytes and exceeds the maximum of "
-        . URL_MAX_LENGTH . ' bytes. Shorten your parameters.';
-    }
-  }
-  else {
-    $response = $self->ua->get( $uri, referer => $self->referer );
-  }
+  my $uri =
+    (defined $langpair ? GOOGLE_TRANSLATE_URL . '&langpair=' . $langpair : GOOGLE_DETECT_URL)
+    . (defined $self->{'key'} ? '&key=' . uri_escape($self->{'key'}) : '')
+    . '&q=' . uri_escape($text);
 
+  my $response = $self->{'ua'}->get($uri);
   if ($response->is_success) {
-    my $result = eval { $self->json->decode($response->content) };
+    my $result = eval { $self->{'json'}->decode($response->content) };
     if ($@) {
-      Carp::croak "Couldn't parse response from '$uri': $@";
+      croak "Couldn't parse response from \"$uri\": $@";
     }
     else {
       return bless $result, 'WebService::Google::Language::Result';
     }
   }
   else {
-    Carp::croak "An HTTP error occured while getting '$uri': " . $response->status_line;
-  }
-}
-
-sub _utf8_encode {
-  if ($] < 5.007) {
-
-    # on Perl 5.6 the JSON 2 module (JSON::PP56) provides the missing
-    # utf8::encode function, but it seems to be broken
-    # my own UTF8 encoder is tested on ActivePerl 5.6.1.638
-
-    if (length $_[0] == do { use bytes; length $_[0] }) {
-      $_[0] = pack 'U*', unpack 'C*', $_[0];
-    }
-  }
-  else {
-    utf8::encode($_[0]);
+    croak "An HTTP error occured while getting \"$uri\": " . $response->status_line . "\n";
   }
 }
 
@@ -228,34 +124,40 @@ sub _utf8_encode {
 package WebService::Google::Language::Result;
 
 sub error {
-  $_[0]->{responseStatus} && $_[0]->{responseStatus} != 200
-    ? { code    => $_[0]->{responseStatus},
-        message => $_[0]->{responseDetails},
+  $_[0]->{'responseStatus'} != 200
+    ? { 'code'    => $_[0]->{'responseStatus'},
+        'message' => $_[0]->{'responseDetails'},
       }
     : undef
 }
 
-sub code { $_[0]->{responseStatus} }
+sub code { $_[0]->{'responseStatus'} }
 
-sub message { $_[0]->{responseDetails} }
+sub message { $_[0]->{'responseDetails'} }
 
 sub translation {
-  $_[0]->{responseData} ? $_[0]->{responseData}{translatedText} : undef
+  defined $_[0]->{'responseData'}
+    ? $_[0]->{'responseData'}{'translatedText'}
+    : undef
 }
 
 sub language {
-  $_[0]->{responseData}
-    ? $_[0]->{responseData}{language} ||
-      $_[0]->{responseData}{detectedSourceLanguage}
+  defined $_[0]->{'responseData'}
+    ? $_[0]->{'responseData'}{'language'} ||
+      $_[0]->{'responseData'}{'detectedSourceLanguage'}
     : undef
 }
 
 sub is_reliable {
-  $_[0]->{responseData} ? $_[0]->{responseData}{isReliable} : undef
+  defined $_[0]->{'responseData'}
+    ? $_[0]->{'responseData'}{'isReliable'}
+    : undef
 }
 
 sub confidence {
-  $_[0]->{responseData} ? $_[0]->{responseData}{confidence} : undef
+  defined $_[0]->{'responseData'}
+    ? $_[0]->{'responseData'}{'confidence'}
+    : undef
 }
 
 
@@ -272,13 +174,13 @@ WebService::Google::Language - Perl interface to the Google AJAX Language API
 
   use WebService::Google::Language;
 
-  my $service = WebService::Google::Language->new(
-    referer => 'http://example.com/',
-    src     => '',
-    dest    => 'en',
+  $service = WebService::Google::Language->new(
+    'referer' => 'http://example.com/',
+    'src'     => '',
+    'dest'    => 'en',
   );
 
-  my $result = $service->translate('Hallo Welt');
+  $result = $service->translate('Hallo Welt');
   if ($result->error) {
     printf "Error code: %s\n", $result->code;
     printf "Message:    %s\n", $result->message;
@@ -312,37 +214,27 @@ Creates a new C<WebService::Google::Language> object and returns it.
 Key/value pair arguments set up the initial state:
 
   Key       Usage         Expected value
-  ---------------------------------------------------------
+  ----------------------------------------------------
   referer   mandatory     HTTP referer
   src       optional      default source language
   dest      optional      default destination language
   key       recommended   application's key
-  ua        optional      an LWP::UserAgent object for reuse
-  json      optional      a JSON object for reuse
 
 Since Google demands a "valid and accurate http referer header" in
 requests to their service, a non-empty referer string must be passed
 to the constructor. Otherwise the constructor will fail.
 
-Unless the key 'ua' contains an instance of C<LWP::UserAgent>, any additional
-entries in the C<%options> hash will be passed unmodified to the constructor
-of C<LWP::UserAgent>, which is used for performing the requests.
+Any additional entries in the C<%options> hash will be passed unmodified
+to the constructor of C<LWP::UserAgent>, which is used for performing
+the requests.
 
 E.g. you can set your own user agent identification and specify a timeout
 this way:
 
   $service = WebService::Google::Language->new(
-    referer => 'http://example.com/',
-    agent   => 'My Application 2.0',
-    timeout => 5,
-  );
-
-Or reuse existing instances of C<LWP::UserAgent> and C<JSON> respectively:
-
-  $service = WebService::Google::Language->new(
-    referer => 'http://example.com/',
-    ua      => $my_ua_obj,
-    json    => $my_json_obj,
+    'referer' => 'http://example.com/',
+    'agent'   => 'My Application 2.0',
+    'timeout' => 5,
   );
 
 =item $service = WebService::Google::Language->new($referer);
@@ -389,13 +281,13 @@ Examples:
   $result = $service->translate('Hallo Welt');
 
   # auto-detect source language and translate to French (fr)
-  $result = $service->translate( 'Hallo Welt', dest => 'fr' );
+  $result = $service->translate('Hallo Welt', 'dest' => 'fr');
 
   # set source to English and destination to German (de)
   %args = (
-    text => 'Hello world',
-    src  => 'en',
-    dest => 'de',
+    'text' => 'Hello world',
+    'src'  => 'en',
+    'dest' => 'de',
   );
   $result = $service->translate(%args);
 
@@ -404,14 +296,12 @@ and valid language translation pairs.
 
 =item $result = $service->detect($text);
 
-=item $result = $service->detect( text => $text );
+=item $result = $service->detect('text' => $text);
 
 The C<detect> method will request the detection of the language of
 a given text. C<$text> is the single parameter and can be passed directly
 or as key 'text' of a hash.
 
-=item $result = $service->detect_language($text);
-
 If C<detect> as a method name is just not descriptive enough, there is
 an alias C<detect_language> available.
 
@@ -422,31 +312,6 @@ Examples:
   # using the more verbose alias
   $result = $service->detect_language('Hallo Welt');
 
-=item $boolean = $service->ping;
-
-Checks if internet access to Google's service is available.
-
-=item $json = $service->json;
-
-Returns the C<JSON> object used by this instance.
-
-=item $service = $service->json($json);
-
-Sets the C<JSON> object to be used by this instance.
-Setters return their instance and can be chained.
-
-=item $ua = $service->ua;
-
-=item $service = $service->ua($ua);
-
-Returns/sets the C<LWP::UserAgent> object.
-
-=item $referer = $service->referer;
-
-=item $service = $service->referer($referer);
-
-Returns/sets the referer string.
-
 =back
 
 =head1 RESULT ACCESSOR METHODS
@@ -474,32 +339,16 @@ C<undef> for a result from C<translate> or C<detect> respectively.
   is_reliable   no       yes    reliability of detected language
   confidence    no       yes    confidence level, ranging from 0 to 1.0
 
-The L<"SYNOPSIS"> of this module includes a complete example of using the
+The L</"SYNOPSIS"> of this module includes a complete example of using the
 accessor methods.
 
 =head1 LIMITATIONS
 
-Google does not allow submission of text exceeding 5000 characters in length
+Google does not allow submission of text exceeding 500 characters in length
 to their service (see Terms of Use). This module will check the length of
 text passed to its methods and will fail if text is too long (without sending
 a request to Google).
 
-=head1 TODO
-
-=over 4
-
-=item * Batch Interface
-
-Incorporate the Batch Interface into this module
-(L<http://code.google.com/apis/ajaxlanguage/documentation/reference.html#_batch_interface>).
-
-=item * Documentation
-
-Explain the server-side length limitation of URLs for GET requests
-which (currently) must be used by the C<detect> method.
-
-=back
-
 =head1 SEE ALSO
 
 =over 4
@@ -526,29 +375,16 @@ L<http://code.google.com/apis/ajaxlanguage/documentation/reference.html>
 
 =head1 AUTHOR
 
-Henning Manske <hma@cpan.org>
-
-=head1 ACKNOWLEDGEMENTS
-
-Thanks to Igor Sutton (IZUT) for submitting a patch to enable the use of
-proxy environment variables within C<LWP::UserAgent>.
-
-Thanks to Ilya Rubtsov for pointing out Google's change of the text length
-limitation (see Terms of Use) and the existing server-side length limitation
-of URLs when using GET request method.
+Henning Manske (henning.manske@googlemail.com)
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2008-2010 Henning Manske. All rights reserved.
+Copyright 2008 Henning Manske, State and University Library Hamburg.
+All rights reserved.
 
-This module is free software. You can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
+L<http://www.sub.uni-hamburg.de/>
 
-See L<http://dev.perl.org/licenses/>.
-
-This module is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+This program is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
 
 =cut
